/**
 *  Creator: Christian Hotz
 *  Company: hydra newmedia GmbH
 *  Date: 12.06.16
 *
 *  Copyright hydra newmedia GmbH
 */
"use strict";
/**
 *  Imports
 */
var mongoose = require('mongoose');
var mongoose_repo_1 = require('mongoose-repo');
var Snapshot_1 = require('../../utils/Snapshot');
var Diff_1 = require('../../utils/Diff');
var Creator_1 = require('../../utils/Creator');
var DiffModel_1 = require('./models/DiffModel');
var SnapshotModel_1 = require('./models/SnapshotModel');
var MongooseStorage = (function () {
    function MongooseStorage(loggerOrCfg, dbConnectionString, dbOptions) {
        if (!mongoose.connection.readyState && dbConnectionString) {
            mongoose.connect(dbConnectionString, dbOptions ? dbOptions : null);
        }
        this.snapshotRepository = new mongoose_repo_1.Repository(SnapshotModel_1.SnapshotCollection, loggerOrCfg);
        this.diffRepository = new mongoose_repo_1.Repository(DiffModel_1.DiffCollection, loggerOrCfg);
    }
    MongooseStorage.prototype.insertSnapshot = function (snapshot, callback) {
        this.snapshotRepository.insert(new SnapshotModel_1.SnapshotModel(snapshot), function (err, model) {
            if (err || !model) {
                callback(err);
            }
            else {
                callback(null, snapshot.clone().setId(model._id.toString()));
            }
        });
    };
    MongooseStorage.prototype.upsertSnapshot = function (snapshot, callback) {
        var updateCondition = { 'metadata.entity': snapshot.entity.name, 'metadata.objId': snapshot.objId };
        this.snapshotRepository.updateByCondition(updateCondition, new SnapshotModel_1.SnapshotModel(snapshot), function (err, model) {
            if (err || !model) {
                callback(err);
            }
            else {
                callback(null, snapshot.clone().setId(model._id.toString()));
            }
        });
    };
    MongooseStorage.prototype.insertDiff = function (diff, callback) {
        this.diffRepository.insert(new DiffModel_1.DiffModel(diff), function (err, model) {
            if (err || !model) {
                callback(err);
            }
            else {
                callback(null, diff.clone().setId(model._id.toString()));
            }
        });
    };
    MongooseStorage.prototype.findLatestSnapshotBefore = function (id, timestamp, entity, callback) {
        var searchCondition = { 'metadata.entity': entity.name, 'metadata.objId': id };
        SnapshotModel_1.SnapshotCollection.findOne(searchCondition).sort({ 'metadata.timestamp': -1 })
            .exec(function (err, model) {
            if (err || !model) {
                callback(err);
            }
            else {
                var creator = new Creator_1.Creator(model.metadata.creator.user, model.metadata.creator.source);
                var timestamp_1 = new Date(Date.parse(model.metadata.timestamp));
                callback(null, new Snapshot_1.Snapshot(model.obj, entity, creator, timestamp_1, model._id.toString()));
            }
        });
    };
    MongooseStorage.prototype.findLatestDiffBefore = function (id, timestamp, entity, callback) {
        var searchCondition = { 'metadata.entity': entity.name, 'metadata.objId': id };
        DiffModel_1.DiffCollection.findOne(searchCondition).sort({ 'metadata.timestamp': -1 })
            .exec(function (err, model) {
            if (err || !model) {
                callback(err);
            }
            else {
                var creator = new Creator_1.Creator(model.metadata.creator.user, model.metadata.creator.source);
                var timestamp_2 = new Date(Date.parse(model.metadata.timestamp));
                callback(null, new Diff_1.Diff(model.obj, id, entity, creator, timestamp_2, model._id.toString()));
            }
        });
    };
    return MongooseStorage;
}());
exports.MongooseStorage = MongooseStorage;
